@@ -15,13 +15,17 @@ Contributors:
 """
 
 import os
-import re
 import cgi
 import cStringIO
 import logging
+from re import compile, sub, escape, DOTALL
 try:
+    # have web2py
     from restricted import RestrictedError
-except:
+    from globals import current
+except ImportError:
+    # do not have web2py
+    current = None
     def RestrictedError(a,b,c):
         logging.error(str(a)+':'+str(b)+':'+str(c))
         return RuntimeError
@@ -47,12 +51,24 @@ class SuperNode(Node):
         if self.value:
             return str(self.value)
         else:
-            raise SyntaxError("Undefined parent block ``%s``. \n" % self.name + \
-"You must define a block before referencing it.\nMake sure you have not left out an ``{{end}}`` tag." )
+            # raise SyntaxError("Undefined parent block ``%s``. \n" % self.name + "You must define a block before referencing it.\nMake sure you have not left out an ``{{end}}`` tag." )
+            return ''
 
     def __repr__(self):
         return "%s->%s" % (self.name, self.value)
 
+def output_aux(node,blocks):
+    # If we have a block level
+    #   If we can override this block.
+    #     Override block from vars.
+    #   Else we take the default
+    # Else its just a string
+    return (blocks[node.name].output(blocks)
+            if node.name in blocks else
+            node.output(blocks)) \
+            if isinstance(node, BlockNode) \
+            else str(node)
+
 class BlockNode(Node):
     """
     Block Container.
@@ -77,8 +93,7 @@ class BlockNode(Node):
 
     def __repr__(self):
         lines = ['%sblock %s%s' % (self.left,self.name,self.right)]
-        for node in self.nodes:
-            lines.append(str(node))
+        lines += [str(node) for node in self.nodes]
         lines.append('%send%s' % (self.left, self.right))
         return ''.join(lines)
 
@@ -86,11 +101,8 @@ class BlockNode(Node):
         """
         Get this BlockNodes content, not including child Nodes
         """
-        lines = []
-        for node in self.nodes:
-            if not isinstance(node, BlockNode):
-                lines.append(str(node))
-        return ''.join(lines)
+        return ''.join(str(node) for node in self.nodes \
+                           if not isinstance(node, BlockNode))
 
     def append(self, node):
         """
@@ -118,30 +130,14 @@ class BlockNode(Node):
         else:
             raise TypeError("Invalid type; must be instance of ``BlockNode``. %s" % other)
 
+
     def output(self, blocks):
         """
         Merges all nodes into a single string.
-
         blocks -- Dictionary of blocks that are extending
         from this template.
         """
-        lines = []
-        # Get each of our nodes
-        for node in self.nodes:
-            # If we have a block level node.
-            if isinstance(node, BlockNode):
-                # If we can override this block.
-                if node.name in blocks:
-                    # Override block from vars.
-                    lines.append(blocks[node.name].output(blocks))
-                # Else we take the default
-                else:
-                    lines.append(node.output(blocks))
-            # Else its just a string
-            else:
-                lines.append(str(node))
-        # Now combine all of our lines together.
-        return ''.join(lines)
+        return ''.join(output_aux(node,blocks) for node in self.nodes)
 
 class Content(BlockNode):
     """
@@ -161,29 +157,13 @@ class Content(BlockNode):
         self.pre_extend = pre_extend
 
     def __str__(self):
-        lines = []
-        # For each of our nodes
-        for node in self.nodes:
-            # If it is a block node.
-            if isinstance(node, BlockNode):
-                # And the node has a name that corresponds with a block in us
-                if node.name in self.blocks:
-                    # Use the overriding output.
-                    lines.append(self.blocks[node.name].output(self.blocks))
-                else:
-                    # Otherwise we just use the nodes output.
-                    lines.append(node.output(self.blocks))
-            else:
-                # It is just a string, so include it.
-                lines.append(str(node))
-        # Merge our list together.
-        return ''.join(lines)
+        return ''.join(output_aux(node,self.blocks) for node in self.nodes)
 
     def _insert(self, other, index = 0):
         """
         Inserts object at index.
         """
-        if isinstance(other, str) or isinstance(other, Node):
+        if isinstance(other, (str, Node)):
             self.nodes.insert(index, other)
         else:
             raise TypeError("Invalid type, must be instance of ``str`` or ``Node``.")
@@ -206,7 +186,7 @@ class Content(BlockNode):
         """
         Adds a node to list. If it is a BlockNode then we assign a block for it.
         """
-        if isinstance(node, str) or isinstance(node, Node):
+        if isinstance(node, (str, Node)):
             self.nodes.append(node)
             if isinstance(node, BlockNode):
                 self.blocks[node.name] = node
@@ -229,18 +209,18 @@ class Content(BlockNode):
 class TemplateParser(object):
 
     default_delimiters = ('{{','}}')
-    r_tag = re.compile(r'(\{\{.*?\}\})', re.DOTALL)
+    r_tag = compile(r'(\{\{.*?\}\})', DOTALL)
 
-    r_multiline = re.compile(r'(""".*?""")|(\'\'\'.*?\'\'\')', re.DOTALL)
+    r_multiline = compile(r'(""".*?""")|(\'\'\'.*?\'\'\')', DOTALL)
 
     # These are used for re-indentation.
     # Indent + 1
-    re_block = re.compile('^(elif |else:|except:|except |finally:).*$',
-                      re.DOTALL)
+    re_block = compile('^(elif |else:|except:|except |finally:).*$',DOTALL)
+                      
     # Indent - 1
-    re_unblock = re.compile('^(return|continue|break|raise)( .*)?$', re.DOTALL)
+    re_unblock = compile('^(return|continue|break|raise)( .*)?$', DOTALL)
     # Indent - 1
-    re_pass = re.compile('^pass( .*)?$', re.DOTALL)
+    re_pass = compile('^pass( .*)?$', DOTALL)
 
     def __init__(self, text,
                  name    = "ParserContainer",
@@ -287,13 +267,16 @@ class TemplateParser(object):
         # allow optional alternative delimiters
         self.delimiters = delimiters
         if delimiters != self.default_delimiters:
-            escaped_delimiters = (re.escape(delimiters[0]),re.escape(delimiters[1]))
-            self.r_tag = re.compile(r'(%s.*?%s)' % escaped_delimiters, re.DOTALL)
-        elif context.has_key('response') and hasattr(context['response'],'delimiters'):
+            escaped_delimiters = (escape(delimiters[0]),
+                                  escape(delimiters[1]))
+            self.r_tag = compile(r'(%s.*?%s)' % escaped_delimiters, DOTALL)
+        elif hasattr(context.get('response',None),'delimiters'):
             if context['response'].delimiters != self.default_delimiters:
-                escaped_delimiters = (re.escape(context['response'].delimiters[0]),
-                                      re.escape(context['response'].delimiters[1]))
-                self.r_tag = re.compile(r'(%s.*?%s)' % escaped_delimiters,re.DOTALL)
+                escaped_delimiters = (
+                    escape(context['response'].delimiters[0]),
+                    escape(context['response'].delimiters[1]))
+                self.r_tag = compile(r'(%s.*?%s)' % escaped_delimiters,
+                                     DOTALL)
 
         # Create a root level Content that everything will go into.
         self.content = Content(name=name)
@@ -430,9 +413,14 @@ class TemplateParser(object):
         if not filename.strip():
             self._raise_error('Invalid template filename')
 
+        # Allow Views to include other views dynamically
+        context = self.context
+        if current and not "response" in context:
+            context["response"] = current.response
+
         # Get the filename; filename looks like ``"template.html"``.
         # We need to eval to remove the quotes and get the string type.
-        filename = eval(filename, self.context)
+        filename = eval(filename, context)
 
         # Get the path of the file on the system.
         filepath = self.path and os.path.join(self.path, filename) or filename
@@ -515,17 +503,19 @@ class TemplateParser(object):
         # the parent nodes.
         self.content.nodes = []
 
+        t_content = t.content
+        
         # Set our include, unique by filename
-        t.content.blocks['__include__' + filename] = buf
+        t_content.blocks['__include__' + filename] = buf
 
         # Make sure our pre_extended nodes go first
-        t.content.insert(pre)
+        t_content.insert(pre)
 
         # Then we extend our blocks
-        t.content.extend(self.content)
+        t_content.extend(self.content)
 
         # Work off the parent node.
-        self.content = t.content
+        self.content = t_content
 
     def parse(self, text):
 
@@ -544,69 +534,20 @@ class TemplateParser(object):
         ij = self.r_tag.split(text)
         # j = current index
         # i = current item
+        stack = self.stack
         for j in range(len(ij)):
             i = ij[j]
 
             if i:
-                if len(self.stack) == 0:
+                if not stack:
                     self._raise_error('The "end" tag is unmatched, please check if you have a starting "block" tag')
 
                 # Our current element in the stack.
-                top = self.stack[-1]
+                top = stack[-1]
 
                 if in_tag:
                     line = i
 
-                    # If we are missing any strings!!!!
-                    # This usually happens with the following example
-                    # template code
-                    #
-                    # {{a = '}}'}}
-                    # or
-                    # {{a = '}}blahblah{{'}}
-                    #
-                    # This will fix these
-                    # This is commented out because the current template
-                    # system has this same limitation. Since this has a
-                    # performance hit on larger templates, I do not recommend
-                    # using this code on production systems. This is still here
-                    # for "i told you it *can* be fixed" purposes.
-                    #
-                    #
-#                    if line.count("'") % 2 != 0 or line.count('"') % 2 != 0:
-#
-#                        # Look ahead
-#                        la = 1
-#                        nextline = ij[j+la]
-#
-#                        # As long as we have not found our ending
-#                        # brackets keep going
-#                        while '}}' not in nextline:
-#                            la += 1
-#                            nextline += ij[j+la]
-#                            # clear this line, so we
-#                            # don't attempt to parse it
-#                            # this is why there is an "if i"
-#                            # around line 530
-#                            ij[j+la] = ''
-#
-#                        # retrieve our index.
-#                        index = nextline.index('}}')
-#
-#                        # Everything before the new brackets
-#                        before = nextline[:index+2]
-#
-#                        # Everything after
-#                        after = nextline[index+2:]
-#
-#                        # Make the next line everything after
-#                        # so it parses correctly, this *should* be
-#                        # all html
-#                        ij[j+1] = after
-#
-#                        # Add everything before to the current line
-#                        line += before
-
                     # Get rid of '{{' and '}}'
                     line = line[2:-2].strip()
 
@@ -624,9 +565,9 @@ class TemplateParser(object):
                     # Perform block comment escaping.
                     # This performs escaping ON anything
                     # in between """ and """
-                    line = re.sub(TemplateParser.r_multiline,
-                                remove_newline,
-                                line)
+                    line = sub(TemplateParser.r_multiline,
+                               remove_newline,
+                               line)
 
                     if line.startswith('='):
                         # IE: {{=response.title}}
@@ -663,7 +604,7 @@ class TemplateParser(object):
                         self.lexers[name](parser    = self,
                                           value     = value,
                                           top       = top,
-                                          stack     = self.stack,)
+                                          stack     = stack)
 
                     elif name == '=':
                         # So we have a variable to insert into
@@ -684,7 +625,7 @@ class TemplateParser(object):
                         # so anything after this gets added
                         # to this node. This allows us to
                         # "nest" nodes.
-                        self.stack.append(node)
+                        stack.append(node)
 
                     elif name == 'end' and not value.startswith('='):
                         # We are done with this node.
@@ -693,7 +634,7 @@ class TemplateParser(object):
                         self.blocks[top.name] = top
 
                         # Pop it.
-                        self.stack.pop()
+                        stack.pop()
 
                     elif name == 'super' and not value.startswith('='):
                         # Get our correct target name
@@ -748,17 +689,17 @@ class TemplateParser(object):
                             # So we can properly put a response.write() in place.
                             continuation = False
                             len_parsed = 0
-                            for k in range(len(tokens)):
+                            for k, token in enumerate(tokens):
 
-                                tokens[k] = tokens[k].strip()
-                                len_parsed += len(tokens[k])
+                                token = tokens[k] = token.strip()
+                                len_parsed += len(token)
 
-                                if tokens[k].startswith('='):
-                                    if tokens[k].endswith('\\'):
+                                if token.startswith('='):
+                                    if token.endswith('\\'):
                                         continuation = True
-                                        tokens[k] = "\n%s(%s" % (self.writer, tokens[k][1:].strip())
+                                        tokens[k] = "\n%s(%s" % (self.writer, token[1:].strip())
                                     else:
-                                        tokens[k] = "\n%s(%s)" % (self.writer, tokens[k][1:].strip())
+                                        tokens[k] = "\n%s(%s)" % (self.writer, token[1:].strip())
                                 elif continuation:
                                     tokens[k] += ')'
                                     continuation = False
@@ -832,6 +773,32 @@ def get_parsed(text):
     """
     return str(TemplateParser(text))
 
+class DummyResponse():
+    def __init__(self):
+        self.body = cStringIO.StringIO()
+    def write(self, data, escape=True):
+        if not escape:
+            self.body.write(str(data))
+        elif hasattr(data,'xml') and callable(data.xml):
+            self.body.write(data.xml())
+        else:
+            # make it a string
+            if not isinstance(data, (str, unicode)):
+                data = str(data)
+            elif isinstance(data, unicode):
+                data = data.encode('utf8', 'xmlcharrefreplace')
+            data = cgi.escape(data, True).replace("'","&#x27;")
+            self.body.write(data)
+
+class NOESCAPE():
+    """
+    A little helper to avoid escaping.
+    """
+    def __init__(self, text):
+        self.text = text
+    def xml(self):
+        return self.text
+
 # And this is a generic render function.
 # Here for integration with gluon.
 def render(content = "hello world",
@@ -868,42 +835,31 @@ def render(content = "hello world",
     >>> render(content='{{for i in range(3):\\n=i\\npass}}')
     '012'
     """
-    # Here to avoid circular Imports
+    # here to avoid circular Imports
     try:
         from globals import Response
-    except:
+    except ImportError:
         # Working standalone. Build a mock Response object.
-        class Response():
-            def __init__(self):
-                self.body = cStringIO.StringIO()
-            def write(self, data, escape=True):
-                if not escape:
-                    self.body.write(str(data))
-                elif hasattr(data,'xml') and callable(data.xml):
-                    self.body.write(data.xml())
-                else:
-                    # make it a string
-                    if not isinstance(data, (str, unicode)):
-                        data = str(data)
-                    elif isinstance(data, unicode):
-                        data = data.encode('utf8', 'xmlcharrefreplace')
-                    data = cgi.escape(data, True).replace("'","&#x27;")
-                    self.body.write(data)
+        Response = DummyResponse
 
-        # A little helper to avoid escaping.
-        class NOESCAPE():
-            def __init__(self, text):
-                self.text = text
-            def xml(self):
-                return self.text
         # Add it to the context so we can use it.
-        context['NOESCAPE'] = NOESCAPE
+        if not 'NOESCAPE' in context:
+            context['NOESCAPE'] = NOESCAPE
+
+    # save current response class
+    if context and 'response' in context:
+        old_response_body = context['response'].body
+        context['response'].body = cStringIO.StringIO()
+    else:
+        old_response_body = None
+        context['response'] = Response()
 
     # If we don't have anything to render, why bother?
     if not content and not stream and not filename:
         raise SyntaxError, "Must specify a stream or filename or content"
 
-    # Here for legacy purposes, probably can be reduced to something more simple.
+    # Here for legacy purposes, probably can be reduced to
+    # something more simple.
     close_stream = False
     if not stream:
         if filename:
@@ -912,9 +868,6 @@ def render(content = "hello world",
         elif content:
             stream = cStringIO.StringIO(content)
 
-    # Get a response class.
-    context['response'] = Response()
-
     # Execute the template.
     code = str(TemplateParser(stream.read(), context=context, path=path, lexers=lexers, delimiters=delimiters))
     try:
@@ -927,7 +880,10 @@ def render(content = "hello world",
         stream.close()
 
     # Returned the rendered content.
-    return context['response'].body.getvalue()
+    text = context['response'].body.getvalue()
+    if old_response_body is not None:
+        context['response'].body = old_response_body
+    return text
 
 
 if __name__ == '__main__':
@@ -937,3 +893,6 @@ if __name__ == '__main__':
 
 
 
+
+
+
